X-Git-Url: https://www.chiark.greenend.org.uk/ucgi/~mdw/git/disorder/blobdiff_plain/0f55e9050dd26ab8804934a188f1ee8ab8b55ffe..985bb670b4e07d35cb1580780253ded2524a342e:/lib/cookies.c
diff --git a/lib/cookies.c b/lib/cookies.c
index 425b84c..0ff17b5 100644
--- a/lib/cookies.c
+++ b/lib/cookies.c
@@ -1,47 +1,39 @@
/*
* This file is part of DisOrder
- * Copyright (C) 2007 Richard Kettlewell
+ * Copyright (C) 2007, 2008 Richard Kettlewell
*
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * General Public License for more details.
- *
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
* You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * along with this program. If not, see .
*/
/** @file lib/cookies.c
* @brief Cookie support
*/
-#include
-#include "types.h"
+#include "common.h"
-#include
-#include
-#include
#include
#include
#include
#include
-#include "rights.h"
#include "cookies.h"
#include "hash.h"
#include "mem.h"
#include "log.h"
#include "printf.h"
-#include "mime.h"
+#include "base64.h"
#include "configuration.h"
#include "kvp.h"
-#include "rights.h"
#include "trackdb.h"
/** @brief Hash function used in signing HMAC */
@@ -83,6 +75,16 @@ static void newkey(void) {
hash_foreach(revoked, revoked_cleanup_callback, &now);
}
+/** @brief Base64 mapping table for cookies
+ *
+ * Stupid Safari cannot cope with quoted cookies, so cookies had better not
+ * need quoting. We use $ to separate the parts of the cookie and +%# to where
+ * MIME uses +/=; see @ref base64.c. See http_separator() for the characters
+ * to avoid.
+ */
+static const char cookie_base64_table[] =
+ "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+%#";
+
/** @brief Sign @p subject with @p key and return the base64 of the result
* @param key Key to sign with (@ref HASHSIZE bytes)
* @param subject Subject string
@@ -106,7 +108,7 @@ static char *sign(const uint8_t *key,
}
gcry_md_write(h, subject, strlen(subject));
sig = gcry_md_read(h, ALGO);
- sig64 = mime_to_base64(sig, HASHSIZE);
+ sig64 = generic_to_base64(sig, HASHSIZE, cookie_base64_table);
gcry_md_close(h);
return sig64;
}
@@ -120,9 +122,9 @@ char *make_cookie(const char *user) {
time_t now;
char *b, *bp, *c, *g;
- /* semicolons aren't allowed in usernames */
- if(strchr(user, ';')) {
- error(0, "make_cookie for username with semicolon");
+ /* dollar signs aren't allowed in usernames */
+ if(strchr(user, '$')) {
+ error(0, "make_cookie for username with dollar sign");
return 0;
}
/* look up the password */
@@ -136,7 +138,7 @@ char *make_cookie(const char *user) {
if(now >= signing_key_validity_limit)
newkey();
/* construct the subject */
- byte_xasprintf(&b, "%jx;%s;", (intmax_t)now + config->cookie_login_lifetime,
+ byte_xasprintf(&b, "%jx$%s$", (intmax_t)now + config->cookie_login_lifetime,
urlencodestring(user));
byte_xasprintf(&bp, "%s%s", b, password);
/* sign it */
@@ -172,12 +174,12 @@ char *verify_cookie(const char *cookie, rights_type *rights) {
error(errno, "error parsing cookie timestamp");
return 0;
}
- if(*c1 != ';') {
+ if(*c1 != '$') {
error(0, "invalid cookie timestamp");
return 0;
}
- /* There'd better be two semicolons */
- c2 = strchr(c1 + 1, ';');
+ /* There'd better be two dollar signs */
+ c2 = strchr(c1 + 1, '$');
if(c2 == 0) {
error(0, "invalid cookie syntax");
return 0;
@@ -202,7 +204,7 @@ char *verify_cookie(const char *cookie, rights_type *rights) {
return 0;
/* construct the expected subject. We re-encode the timestamp and the
* password. */
- byte_xasprintf(&bp, "%jx;%s;%s", t, urlencodestring(user), password);
+ byte_xasprintf(&bp, "%jx$%s$%s", t, urlencodestring(user), password);
/* Compute the expected signature. NB we base64 the expected signature and
* compare that rather than exposing our base64 parser to the cookie. */
if(!(sig = sign(signing_key, bp)))
@@ -234,7 +236,7 @@ void revoke_cookie(const char *cookie) {
/* reject bogus cookies */
if(errno)
return;
- if(*ptr != ';')
+ if(*ptr != '$')
return;
/* make sure the revocation list exists */
if(!revoked)